#region Maths Expression Parser
// Based on article:
// Evaluating Mathematical Expressions by Compiling C# Code at Runtime
// By Vlad Tepes 
// http://www.codeproject.com/csharp/CustExprEval.asp
#endregion

using System;
using System.Reflection;
using System.Windows.Forms;
using Microsoft.CSharp;
using System.CodeDom.Compiler;

namespace Example1 {
	
	/// <summary>
	/// This is the base evaluator class
	/// </summary>
	abstract public class BaseEvaluator {  
		abstract public double Eval(double x,double y);
	}

	public class MathExpressionParser {

		BaseEvaluator myobj = null; 
		public string GeneratedSource;

		public MathExpressionParser()  {  
		}  
		
		/// <summary>
		/// This method generates a new evaluation object using the passed in expression.
		/// </summary>
		public bool Initialize(string expr)  {   
			// Obtain access to a C# Provider
			CSharpCodeProvider cp = new CSharpCodeProvider();   
			// Then get hold of a compiler interface
			ICodeCompiler ic = cp.CreateCompiler(); 
			// Create a parameters object
			CompilerParameters cpar	= new CompilerParameters();   
			cpar.GenerateInMemory = true;   
			cpar.GenerateExecutable = false;   
			cpar.ReferencedAssemblies.Add("system.dll");   
			cpar.ReferencedAssemblies.Add("Example1.exe");    
			GeneratedSource = 
				"using System;\r\n"
				+ "using Example1;\r\n"
				+ "namespace Example1 {\r\n"
				+ "  class CompiledEvaluator : BaseEvaluator {\r\n" 
				+ "    public override double Eval(double x,double y) {\r\n"
				+	"      return " + expr + ";\r\n"
				+ "    }\r\n"
				+ "  }\r\n"
				+ "}\r\n";   

			// Now we can do the compile, get the results back into an object
			CompilerResults cr = ic.CompileAssemblyFromSource(cpar,GeneratedSource);   
			// Check the errors object for any issues
			foreach (CompilerError ce in cr.Errors)    
				MessageBox.Show(ce.ErrorText);    
			// If we don't have any errors and we have a valid compiled assembly then we can
			// instantiate it 
			if (cr.Errors.Count == 0 && cr.CompiledAssembly != null)   {    
				Type ObjType = cr.CompiledAssembly.GetType("Example1.CompiledEvaluator");    
				try    {     
					if (ObjType != null)     {      
						myobj = (BaseEvaluator)Activator.CreateInstance(ObjType);     
					}    
				}    
				catch (Exception ex)    {     
					MessageBox.Show(ex.Message);    
				}    
				return true;   
			} else     
				return false;  
		}  

		// Now do the evaluation using the private object
		public double Evaluate(double x,double y)  {   
			double val = 0.0;   
			if (myobj != null)   {     
				val = myobj.Eval(x,y);   }   
			return val;  
		} 
	}	
}
