using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Reflection;
using Example2a.Framework;
using Example2a.Domain;

namespace Example2a.Test {

	public class TestForm : System.Windows.Forms.Form {
		private System.Windows.Forms.Button ButtonTestNull;
		private System.Windows.Forms.Button ButtonTest10000;
		private System.Windows.Forms.Button ButtonTest10000Reflection;

		public TestForm() {
			InitializeComponent();
			Engine.Initialize();
		}

		private void ButtonTestNull_Click(object sender, System.EventArgs e) {
			SimpleObject target = new SimpleObject();
			bool isValid = Engine.IsValid(target);
			MessageBox.Show(String.Format("Object is valid = {0}", isValid));
		}
		private void ButtonTest10000_Click(object sender, System.EventArgs e) {
			DateTime start = DateTime.Now;
			SimpleObject target = new SimpleObject();
			for (int i=0; i < 1000000 ; i++) {
				bool isValid = Engine.IsValid(target);
			}
			DateTime finish = DateTime.Now;
			TimeSpan elapsed = finish.Subtract(start);
			MessageBox.Show(String.Format("Time for 1,000,000 = {0} seconds", elapsed.TotalSeconds.ToString()));
		}
		private void ButtonTest10000Reflection_Click(object sender, System.EventArgs e) {
			DateTime start = DateTime.Now;
			SimpleObject target = new SimpleObject();
			for (int i=0; i < 1000000 ; i++) {
				bool isValid = IsValid(target);
			}
			DateTime finish = DateTime.Now;
			TimeSpan elapsed = finish.Subtract(start);
			MessageBox.Show(String.Format("Time for 1,000,000 = {0} seconds", elapsed.TotalSeconds.ToString()));
		}

		private bool IsValid(object targetObject) {
			Type myType = targetObject.GetType();
			FieldInfo info = myType.GetField("Name");
			object val = info.GetValue(targetObject);
			return (val != null);
		}


		#region Windows Stuff
		private System.ComponentModel.Container components = null;
		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if(components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		[STAThread]
		static void Main() {
			Application.Run(new TestForm());
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			this.ButtonTestNull = new System.Windows.Forms.Button();
			this.ButtonTest10000 = new System.Windows.Forms.Button();
			this.ButtonTest10000Reflection = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// ButtonTestNull
			// 
			this.ButtonTestNull.Location = new System.Drawing.Point(40, 30);
			this.ButtonTestNull.Name = "ButtonTestNull";
			this.ButtonTestNull.Size = new System.Drawing.Size(230, 30);
			this.ButtonTestNull.TabIndex = 0;
			this.ButtonTestNull.Text = "Compiled Test Null";
			this.ButtonTestNull.Click += new System.EventHandler(this.ButtonTestNull_Click);
			// 
			// ButtonTest10000
			// 
			this.ButtonTest10000.Location = new System.Drawing.Point(40, 80);
			this.ButtonTest10000.Name = "ButtonTest10000";
			this.ButtonTest10000.Size = new System.Drawing.Size(230, 30);
			this.ButtonTest10000.TabIndex = 1;
			this.ButtonTest10000.Text = "Test 1,000,000 Compiled";
			this.ButtonTest10000.Click += new System.EventHandler(this.ButtonTest10000_Click);
			// 
			// ButtonTest10000Reflection
			// 
			this.ButtonTest10000Reflection.Location = new System.Drawing.Point(40, 130);
			this.ButtonTest10000Reflection.Name = "ButtonTest10000Reflection";
			this.ButtonTest10000Reflection.Size = new System.Drawing.Size(230, 30);
			this.ButtonTest10000Reflection.TabIndex = 2;
			this.ButtonTest10000Reflection.Text = "Test 1,000,000 Reflection";
			this.ButtonTest10000Reflection.Click += new System.EventHandler(this.ButtonTest10000Reflection_Click);
			// 
			// TestForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(9, 20);
			this.ClientSize = new System.Drawing.Size(292, 270);
			this.Controls.Add(this.ButtonTest10000Reflection);
			this.Controls.Add(this.ButtonTest10000);
			this.Controls.Add(this.ButtonTestNull);
			this.Font = new System.Drawing.Font("Tahoma", 12F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.Name = "TestForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "TestForm";
			this.ResumeLayout(false);

		}
		#endregion

		#endregion

	}
}
