using System;
using System.IO;
using System.Text;

namespace Example2b.Framework {

	public class Generator {
	
		private string template;
		private StringBuilder setsb = new StringBuilder();
		private StringBuilder rulesb = new StringBuilder();
		private SourceRuleSet ruleset;
		private object[] parameters = new object[4];
		
		public Generator() {
			template = ReadFile("RuleTemplate.cs.txt");
		}

		public string GenerateSource(SourceRuleSet source) {
			// Store reference to the rule set
			ruleset = source;
			// Next we have sort out the namespaces and the using clauses
			AppendUsingStatements();
			AppendHeader();
			AppendRuleSetHeader();
			// Construct the name space
			// Now generate the source for the individual rules
			foreach (SourceRule rule in ruleset.Rules) {
				GenerateRuleSource(rule);
			}
			// Put in the rule set footers
			AppendRuleSetFooter();
			// Now merge in the individual rule objects
			setsb.Append(rulesb.ToString());
			// Then put in the final footers
			AppendFooter();

			// Return the full source to the caller
			return setsb.ToString();
			
		}
		protected void GenerateRuleSource(SourceRule rule) {
			GenerateRuleSource(rule, template);
		}

		protected void GenerateRuleSource(SourceRule rule, string template) {
			// First we should get the parameters
			LoadFormatParameters(rule);
			// Construct the rule source
			rulesb.AppendFormat(template, parameters);
			// Now add the entry into the list 
			setsb.Append("      Add( new Rule" + parameters[0] + "());\r\n");
		}

		private void AppendUsingStatements() {
			setsb.Append("using System;\r\n");
			setsb.Append("using Example2b.Framework;\r\n");
			string[] usinglist = ruleset.Using.Split(new char[] {','});
			foreach (string s in usinglist) {
				setsb.Append("using " + s + ";\r\n");
			}
		}
		protected void AppendHeader() {
			setsb.Append("namespace " + ruleset.Namespace + "{\r\n");
		}
		protected void AppendRuleSetHeader() {
			// Now construct the rule list object 
			setsb.Append("  class RuleSetTest : CompiledRuleSet {\r\n");
			setsb.Append("    public override void Initialize() {\r\n");
		}
		protected void AppendRuleSetFooter() {
			setsb.Append("    }\r\n");	// End of the rulelist load method
			setsb.Append("  }\r\n");	// End of the rulelist class
		}
		protected void AppendFooter() {
			setsb.Append("}\r\n");	// End the namespace
		}

		private void LoadFormatParameters(SourceRule rule) {
			parameters[0] = rule.Id.ToString();
			parameters[1] = (rule.Message != null) ? rule.Message : String.Empty;
			parameters[2] = rule.TargetType;
			parameters[3] = rule.Expression;
		}

		private string ReadFile(string filename) {
			StreamReader sr = new StreamReader(filename);
			return sr.ReadToEnd();
		}


	}
}
